<h1 align="center">
  <img width="700" src="https://rawgit.com/codsen/bitsausage/master/media/sausage_v2.png" alt="bitSausage">
  <br>
</h1>

> bitSausage keeps bitHound happy - a command line tool to automate your .bithoundrc's config

[![Link to npm page][npm-img]][npm-url]
[![Build Status][travis-img]][travis-url]
[![bitHound Score][overall-img]][overall-url]
[![bitHound Dependencies][deps-img]][deps-url]
[![bitHound Dev Dependencies][dev-img]][dev-url]
[![Known Vulnerabilities][vulnerabilities-img]][vulnerabilities-url]
[![Downloads/Month][downloads-img]][downloads-url]
[![View dependencies as 2D chart][deps2d-img]][deps2d-url]

## Table of Contents

<!-- START doctoc generated TOC please keep comment here to allow auto update -->
<!-- DON'T EDIT THIS SECTION, INSTEAD RE-RUN doctoc TO UPDATE -->


- [Install & Use](#install--use)
- [What it does](#what-it-does)
- [What it does exactly](#what-it-does-exactly)
- [A nifty setup idea](#a-nifty-setup-idea)
- [Contributing](#contributing)
- [Licence](#licence)

<!-- END doctoc generated TOC please keep comment here to allow auto update -->

## Install & Use

```sh
$ npm i -g bitsausage
```

Install globally. To use it, just call it from your command line in the root of your project:

```sh
$ bitsausage
```

It is silent and minimal; it is meant to run automatically and often.

**[⬆ &nbsp;back to top](#)**

## What it does

`bitSausage` helps to keep [bitHound](https://www.bithound.io/), the code analysis service, happy. `bitSausage` automatically refreshes and maintains the bitHound config files (`.bithoundrc`'s), so you never get a surprise bad scores.

The idea is, if I set some dependencies (especially dev-ones) to be `*`, I mean it. I don't want to edit `.bithoundrc` and I especially don't want my score slashed just because I forgot to add few dep ignores in the config.

This library will generate a correct `.bithoundrc` for your project (or nothing if your project does not use bitHound), then it will write the `.bithoundrc` to the root of your folder. No matter did it already exist or not.

The idea is, a) `bitSausage` should do nothing on projects that don't use bitHound; b) bitSausage should generate the most correct bitHound config for your project's **current state**.

If you chain the `bitSausage` with a common command alias like `git add .`, you can forget to ever maintain bitHound configs.

🍴🍽👅😋🐷😍🐽🌭👌😂

**[⬆ &nbsp;back to top](#)**

## What it does exactly

**Step 0.**

`bitSausage` will first look for `README` file, does it contain a string `bithound` (case not sensitive). If you use the bitHound service, you will surely put some badges which will contain this string. To bypass this check, call `bitSausage` with a flag `--force` or `-f`.

**Next steps.**

When activated, `bitSausage` will perform the following:

1. Scans the `package.json` and detects the **linting setup**. Then fills `critics` key within `.bithoundrc` accordingly.
2. Scans `package.json`, looks for **dependencies requesting `*`** ("_any version_") and replaces the `.bithoundrc` `dependencies` key value's contents with the list of those dependencies. Now bitHound won't swear about requiring "the latest" any more.
3. Scans the package's root folder and its subfolders, **finds all test files** and fills the list in the `.bithoundrc` `test` key's value.
4. Scans the `package.json` npm scripts, scan all script keys, check their values, does any field reference `babel` and has `--out-dir` or `--out-file` on the same row. If it does, parse the next argument of each (if such exists) and put it into array of ignores. This means no semicolon build transpiled files will not get linted and flagged as erroneous by the BitHound.

If `bitSausage` can't find the README, or `package.json`, it will exit with a zero exit code (successful), so it can be safely run on any folder, even the one who doesn't contain `bitHound` projects.

**[⬆ &nbsp;back to top](#)**

## A nifty setup idea

It would be tedious and unnecessary to run `bitsausage` manually. Not to mention, you might even forget to run it.

What I suggest, add `bitSausage` to one of your aliases, for example, `git add .`. That's what I do.

Edit your `.zshrc` (or Bash config, or whatever-you-are-using-shell's config) file to contain:

```
# create a function which runs commands if certain files exist, and skips if they don't:
my-git-add() {
  [ -e readme.md ] && doctoc readme.md
  [ -e readme.md ] && bitsausage
  [ -e changelog.md ] && chlu changelog.md
  npm-check
  git add .
}

# create alias for your command, call the function:
alias gaa=my-git-add
```

The example above runs:

- [doctoc](https://www.npmjs.com/package/doctoc) on `readme.md` if it exists,
- then runs [chlu](https://www.npmjs.com/package/chlu-cli) on `changelog.md` if it exists,
- then runs `bitsausage` (this library),
- then runs [npm-check](https://www.npmjs.com/package/npm-check) and lastly,
- runs the `git add .`.

It means, you always get your documentation committed in a correct, updated state. This example above is growing; I want to automate _everything_. Literally.

😋👅🍽🐷🐽😍🍴🌭👌😂

**[⬆ &nbsp;back to top](#)**

## Contributing

If you see anything incorrect whatsoever, do raise an issue [here on GitHub](https://github.com/codsen/bitsausage/issues). If you have any comments on the code quality, including ideas how to improve something, do advise me (by email, Twitter or in person).

**[⬆ &nbsp;back to top](#)**

## Licence

> MIT License (MIT)

> Copyright (c) 2017 Codsen Ltd, Roy Revelt

> Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

> The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

> THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.

[npm-img]: https://img.shields.io/npm/v/bitsausage.svg
[npm-url]: https://www.npmjs.com/package/bitsausage

[travis-img]: https://travis-ci.org/codsen/bitsausage.svg?branch=master
[travis-url]: https://travis-ci.org/codsen/bitsausage

[overall-img]: https://www.bithound.io/github/codsen/bitsausage/badges/score.svg
[overall-url]: https://www.bithound.io/github/codsen/bitsausage

[deps-img]: https://www.bithound.io/github/codsen/bitsausage/badges/dependencies.svg
[deps-url]: https://www.bithound.io/github/codsen/bitsausage/master/dependencies/npm

[dev-img]: https://www.bithound.io/github/codsen/bitsausage/badges/devDependencies.svg
[dev-url]: https://www.bithound.io/github/codsen/bitsausage/master/dependencies/npm

[downloads-img]: https://img.shields.io/npm/dm/bitsausage.svg
[downloads-url]: https://www.npmjs.com/package/bitsausage

[vulnerabilities-img]: https://snyk.io/test/github/codsen/bitsausage/badge.svg
[vulnerabilities-url]: https://snyk.io/test/github/codsen/bitsausage

[deps2d-img]: https://img.shields.io/badge/deps%20in%202D-see_here-08f0fd.svg
[deps2d-url]: http://npm.anvaka.com/#/view/2d/bitsausage
